"""
netclusterWH
------------

A distributed compute network where every client is also a worker.
Uses WebSockets for communication between clients and the coordinator server.
"""

__version__ = "0.1.0"
